@@ -9,7 +9,7 @@ use Config::JFDI;
 use Carp qw/croak/;
 use Data::Dumper;
 
-our $VERSION = '0.13';
+our $VERSION = '0.15';
 
 subtype MyConf => as 'HashRef';
 coerce 'MyConf'
@@ -42,9 +42,6 @@ has conf => ( is => 'rw',
                   $self->_parent_field($self->config->{backend}->{args}->{parent_field})
                       if defined $self->config->{backend}->{args}->{parent_field};
 
-                  $self->_sync($self->config->{backend}->{args}->{sync})
-                      if defined $self->config->{backend}->{args}->{sync};
-
                   $self->_load_backend if ! $self->can('backend');
                 }
                 # it's a hash
@@ -76,11 +73,6 @@ has '_backend' => (
                 isa      => 'Str',
                );
 
-has '_sync' => (
-                is       => 'rw',
-                isa      => 'Str',
-               );
-
 has _plugin_ns => (
                 is       => 'rw',
                 required => 1,
@@ -88,10 +80,16 @@ has _plugin_ns => (
                 default  => sub{ 'Backend' },
                   );
 
-has _type_field => (
+has _source_field => (
                 is       => 'rw',
                 isa      => 'Str',
-                default  => sub{ 'type' },
+                default  => sub{ 'source' },
+                  );
+
+has _default_source => (
+                is       => 'rw',
+                isa      => 'Str',
+                default  => sub{ 'Page' },
                   );
 
 has _search_field => (
@@ -106,17 +104,6 @@ has _parent_field => (
                 default  => sub{ 'parent' },
                   );
 
-has _position_field => (
-                is       => 'rw',
-                isa      => 'Str',
-                default  => sub{ 'position' },
-                  );
-
-has _rules => (
-                isa      => "HashRef",
-                is       => "rw",
-);
-
 has tree => (
                 isa      => "HashRef",
                 is       => "rw",
@@ -138,7 +125,6 @@ sub _load_backend {
   $self->_log("Loading $backend backend ...");
   $self->load_plugin( $backend );
   $self->_load_tree;
-  $self->_load_rules if ( defined $self->config->{rules});
   $self->_build_tree;
 }
 
@@ -166,113 +152,95 @@ sub _load_tree {
   $self->tree($self->_load);
 }
 
-# Load rules from config
-sub _load_rules {
-  my $self = shift;
-  $self->_rules($self->config->{'rules'});
-}
-
-# Build Tree (children, position, ...)
-# parents and children become HashRef
+# Build Tree
+# build root and relationships (just belongs_to)
 sub _build_tree {
   my $self = shift;
 
   my $tree = $self->tree;
-  foreach my $id ( sort { $a <=> $b } keys %$tree ) {
-      my $node        = $tree->{$id};
-      $node->{id}     = $id;
-      my $parent      = $tree->{$node->{parent}};
-      $node->{parent} = $parent;
 
-      $self->add($node, $parent);
+  foreach my $id ( sort { $a cmp $b } keys %$tree ) {
+
+
+      my $node = $self->tree->{$id};
+
+      # transform obj_id to object reference
+      foreach my $col ( keys %$node ) {
+
+          # hash = belongs_to
+          if ( ref($node->{$col}) eq 'HASH') {
+
+              my @values = keys %{$node->{$col}};
+              my $value = $values[0];
+
+              if ( $value =~ m/^\w*_\d*$/ ) {
+                  $node->{$col} = $self->tree->{$value};
+              }
+              else {
+                  die "Error: Relationship belongs_to is not well formatted : $value, use Obj_ID as form"
+              }
+          }
+          # array = has_may
+          elsif ( ref($node->{$col}) eq 'ARRAY') {
+              my $newarray = [];
+              foreach my $child (@{$node->{$col}}) {
+                  if ( $child =~ m/^\w*_\d*$/ ) {
+                      push(@$newarray, $self->tree->{$child});
+                  }
+                  elsif ( ref($child) eq 'HASH') {
+                      push(@$newarray, $child);
+                  }
+                  else {
+                      die "Error : has_many relationship $col (=$child) is not formatted, ex: 'Page_3'\n";
+                  }
+              }
+              $node->{$col} = $newarray;
+          }
+      }
+      my $parent_field;
+      if ( defined $self->config && defined $self->config->{backend}->{args}->{sources_name}->{$node->{source}}->{parent_field}) {
+          $parent_field = $self->config->{backend}->{args}->{sources_name}->{$node->{source}}->{parent_field};
+      }
+      else {
+          $parent_field = 'parent';
+      }
+      $self->add($node, $node->{$parent_field});
   }
 }
 
-# return the last node sorted by id
-sub _last_node {
-  my $self = shift;
+# sub  _bckd_create {
+#     my $self = shift;
+#     my $node = shift;
+#     my $msg  = shift;
 
-  my @nodes_sorted_by_id = sort { $a <=> $b } map $_->{id}, values %{$self->tree};
-  return { id => 0 } if ! defined $nodes_sorted_by_id[0];
-  return $self->tree->{$nodes_sorted_by_id[-1]};
-}
+#     return if ( ! $self->_backend || ! $self->_sync );
+#     $self->_log("[" . $self->_backend . "] CREATE " . $node->{name} . " | $msg");
+#     $self->_create($node);
+# }
 
-sub  _bckd_create {
-    my $self = shift;
-    my $node = shift;
-    my $msg  = shift;
+# sub  _bckd_delete {
+#     my $self  = shift;
+#     my $nodes = shift;
+#     my $msg   = shift;
 
-    return if ( ! $self->_backend || ! $self->_sync );
-    $self->_log("[" . $self->_backend . "] CREATE " . $node->{name} . " | $msg");
-    $self->_create($node);
-}
+#     return if ( ! $self->_backend || ! $self->_sync );
+#     my @nodes_name = map { $_->{$self->_search_field} } @$nodes;
+#     $self->_log("[" . $self->_backend . "] DELETE @nodes_name | $msg");
 
-sub  _bckd_delete {
-    my $self  = shift;
-    my $nodes = shift;
-    my $msg   = shift;
+#     $self->_delete($nodes);
+# }
 
-    return if ( ! $self->_backend || ! $self->_sync );
-    my @nodes_name = map { $_->{$self->_search_field} } @$nodes;
-    $self->_log("[" . $self->_backend . "] DELETE @nodes_name | $msg");
+# sub  _bckd_update {
+#     my $self = shift;
+#     my $node = shift;
+#     my $msg  = shift;
 
-    $self->_delete($nodes);
-}
-
-sub  _bckd_update {
-    my $self = shift;
-    my $node = shift;
-    my $msg  = shift;
-
-    return if ( ! $self->_backend || ! $self->_sync );
-    $self->_log("[" . $self->_backend . "] UPDATE " . $node->{name} . " | $msg");
-    $self->_update($node);
-}
+#     return if ( ! $self->_backend || ! $self->_sync );
+#     $self->_log("[" . $self->_backend . "] UPDATE " . $node->{name} . " | $msg");
+#     $self->_update($node);
+# }
 
 
-sub _check_rules{
-    my ($self, $node, $parent) = @_;
-
-    $node->{id} = $self->_last_node->{id} +1
-        if ( ! defined $node->{id} );
-
-    return 1 if (  ! defined $self->config ||
-                   ! defined $self->config->{rules} ||
-                   $node->{id} == 1
-                );
-    # Is node can 'link' to parent ?
-    my $rules = $self->_rules;
-    if ( defined $node->{$self->_type_field} ) {
-        if ( defined $rules->{$node->{$self->_type_field}}) {
-            if ( defined $rules->{$node->{$self->_type_field}}->{$parent->{$self->_type_field}}) {
-                return $rules->{$node->{$self->_type_field}}->{$parent->{$self->_type_field}};
-            }
-            else { return 0 }
-        }
-        else { return 0 }
-    }
-    else { return 0 }
-}
-
-sub _update_children_position {
-    my $self     = shift;
-    my @children = @_;
-
-    my $n=1;
-    foreach my $child ( @children ) {
-        if (  ! defined $child->{$self->_position_field}  ||
-                   $child->{$self->_position_field} != $n )
-            {
-                my $pos = defined $child->{$self->_position_field} ?
-                    $child->{$self->_position_field} : '?';
-                my $msg = 'updating position of '. $child->{$self->_search_field} .
-                    "( $pos -> $n)";
-                $child->{$self->_position_field} = $n;
-                $self->_bckd_update($child, $msg );
-            }
-        $n++;
-    }
-}
 
 
 # removes the child's father
@@ -313,35 +281,44 @@ sub _clone_node {
     return $clone;
 }
 
+
+sub _key_obj {
+  my $self = shift;
+  my $node = shift;
+
+  return $node->{source} . '_' . $node->{id};
+}
+
+
 # return key name of children parent
 sub _key_children {
     my ($self, $node, $parent) = @_;
 
     my $key_children;
-    #if parent have a type
-    if ( defined $parent->{$self->_type_field}) {
+    #if parent have a source
+    if ( defined $parent->{$self->_source_field}) {
 
-        # if node have a type
-        if ( defined $node->{$self->_type_field}) {
+        # if node have a source
+        if ( defined $node->{$self->_source_field}) {
 
-            # if node and parent have the same type
-            if ( $node->{$self->_type_field} eq $parent->{$self->_type_field} ) {
+            # if node and parent have the same source
+            if ( $node->{$self->_source_field} eq $parent->{$self->_source_field} ) {
                 $key_children = 'children';
             }
             else {
-                $key_children = 'children_' . $node->{$self->_type_field};
+                $key_children = 'children_' . $node->{$self->_source_field};
             }
         }
-        # else node have the same type as parent
+        # else node have the same source as parent
         else {
-            $node->{$self->_type_field} = $parent->{$self->_type_field};
+            $node->{$self->_source_field} = $parent->{$self->_source_field};
             $key_children = 'children';
         }
     }
-    # parent haven't type
+    # parent haven't source
     else {
-        if ( defined $node->{$self->_type_field}) {
-            die "node " . $node->{$self->_search_field} . " [id:". $node->{id} . "] have a type but not the parent !";
+        if ( defined $node->{$self->_source_field}) {
+            die "node " . $node->{$self->_search_field} . " [id:". $node->{id} . "] have a source but not the parent [id:" . $parent->{id} . "]!";
         }
         else {
             $key_children = 'children';
@@ -349,14 +326,21 @@ sub _key_children {
     }
     return $key_children;
 }
+
 sub search {
-  my ( $self, $args, $opts ) = @_;
+  my ( $self, $args ) = @_;
 
   my $results = [];
   my $tree = $self->tree;
-  foreach my $id  ( sort {$a <=> $b} keys %$tree ) {
+  foreach my $id  ( sort {$a cmp $b} keys %$tree ) {
+
+      my $found = 1;
+
+      # next search source field and it's not same source'
+      next if ( $args->{$self->_source_field}
+                    && $tree->{$id}->{$self->_source_field} ne $args->{$self->_source_field}
+                );
 
-    my $found = 1;
     foreach my $key ( keys %$args ) {
       my $current;
       if ( $key =~ m/(.*)\.(.*)/) {
@@ -371,7 +355,7 @@ sub search {
               $current = $tree->{$id}->{$key};
           }
           else {
-              die "'$key' is not a key hash [node id:$id]";
+              die "'$key' is not a key hash [node obj_key:$id]";
           }
       }
       my $value = $args->{$key};
@@ -400,12 +384,18 @@ sub search {
 
 
 # ex : search_path(/A/B/C')
+#      or search_path(/A/B/C, { by => 'name', source => 'Page'} )
 sub search_path {
   my ( $self, $path, $opts ) = @_;
 
+
   # search by 'name' if not defined
   $opts->{by} = $self->_search_field if ! defined $opts->{by};
 
+  # source => 'Page' if not defined
+  $opts->{$self->_source_field} = $self->_default_source if ! defined $opts->{$self->_source_field};
+
+
   croak "path must be start by '/' !: $!\n" if ( $path !~ m|^/| );
 
   my $nodes = [ split m%/%, $path ];
@@ -414,8 +404,9 @@ sub search_path {
   my (@found, @not_found);
   my $parent = '/';
   foreach my $node ( @$nodes ) {
-    my $args = { $opts->{by} => $node, "parent\.$opts->{by}" => $parent};
-    my $result = $self->search($args, $opts);
+
+    my $args = { $opts->{by} => $node, "parent\.$opts->{by}" => $parent, source => $opts->{source}};
+    my $result = $self->search( $args );
 
     $parent = $result->{$opts->{by}} if $result;
 
@@ -509,92 +500,52 @@ sub del {
       $self->_remove_child_father($node);
 
       my $key_children = $self->_key_children($node, $node->{parent});
-      # if position field exist, recalc all children position
-      $self->_update_children_position( @{$father->{$key_children}} )
-          if ( defined $node->{$self->_position_field});
 
       # traverse child branches and delete it
       my $nodes = $self->traverse($node);
-      push(@deleted,map { delete $self->tree->{$_->{id}} } @$nodes);
-      $self->_bckd_delete($nodes, "delete " . @$nodes . " node(s)");
+      push(@deleted,map { delete $self->tree->{$self->_key_obj($_)} } @$nodes);
+      #$self->_bckd_delete($nodes, "delete " . @$nodes . " node(s)");
   }
   return @deleted;
 }
 
-# Inserts a node beneath the parent at the given position.
+# Inserts a node beneath the parent
 sub add {
-  my ($self, $node, $parent, $position) = @_;
+  my ($self, $node, $parent) = @_;
 
-  $node->{id}     = $self->_last_node->{id} +1
-      if ! defined $node->{id};
-
-  if ( ! $position && defined $node->{$self->_position_field}) {
-      $position = $node->{$self->_position_field};
-  }
 
   if ( ! $parent ){
       if ( $self->root) {
-          die "root already exist !";
+          die "root already exist [ node: " .$node->{$self->_source_field}. '_' . $node->{id} . " ]!";
       }
       $self->root($node);
   }
 
   $node->{parent} = $parent;
 
-  # check rules
-  return 0 if ! $self->_check_rules($node, $parent);
-
-
-  my $is_exist_in_backend;
-  $is_exist_in_backend = 1
-      if $self->tree->{$node->{id}};
-
   # save ref node in tree
-  $self->tree->{$node->{id}} = $node;
+  $self->tree->{$self->_key_obj($node)} = $node;
 
   my $key_children;
   my $update_children = 0;
+
   # node != root
   if ( defined $parent && $parent ) {
-
       $key_children = $self->_key_children($node, $parent);
 
       if ( $parent ) {
-          if ( $position ) {
-              splice @{$parent->{$key_children}}, $position -1, 0, $self->tree->{$node->{id}};
-              $update_children = 1;
-          }
-          else {
-              push(@{$parent->{$key_children}}, $self->tree->{$node->{id}});
-              if ( defined ${$parent->{$key_children}}[0] && ${$parent->{$key_children}}[0]->{$self->_position_field}) {
-                  $update_children = 1;
-              }
-          }
+          push(@{$parent->{$key_children}}, $node);
       }
   }
-  # add node in tree
-  $self->_bckd_create($node, 'add node ' . $node->{$self->_search_field})
-      if ! $is_exist_in_backend;
+  # add node in backend
+  #$self->_bckd_create($node, 'add node ' . $node->{$self->_search_field})
+  #    if ! $is_exist_in_backend;
 
   return $self->root if ! $parent;
 
-  # recalc position of children
-  $self->_update_children_position(@{$parent->{$key_children}}) if $update_children;
-
   return $node;
 }
 
-sub insert_before {
-  my ($self, $sibling, $node) = @_;
-
-  if ( ! defined $sibling->{$self->_position_field}) {
-      return $self->add($node, $sibling->{parent}, 1 );
-  }
-
-  return $self->add($node, $sibling->{parent}, $sibling->{$self->_position_field} );
-}
-
-
 sub update {
   my ($self, $node, $datas) = @_;
 
@@ -612,22 +563,7 @@ sub update {
       my $children = $parent->{$key_children};
 
 
-      # if update 'position'
-      if ( $k eq $self->_position_field ) {
-          # delete child in parent in previous position
-          my $old = splice(@$children, $previous -1, 1);
-          # add child in parent in new positioh
-          splice(@$children,$datas->{$k} -1,0, $old);
-          $self->_update_children_position(@{$node->{parent}->{$key_children}})
-              if ( defined $node->{$self->_position_field});
-      }
-      elsif ( $k eq $self->_type_field) {
-          # check rules
-          return 0 if ! $self->_check_rules({ type => $datas->{$k} }, $node->{parent});
-      }
-      elsif ( $k eq 'parent') {
-
-          return 0 if ! $self->_check_rules($node, $datas->{parent});
+      if ( $k eq 'parent') {
 
           my $old = $self->_remove_child_father($node);
           my $new_parent = $datas->{$k};
@@ -638,12 +574,7 @@ sub update {
           my $msg = 'updating parent of '. $node->{$self->_search_field} .
               " ( " . $old->{name} . " -> ". $node->{parent}->{name} .")";
 
-          $self->_bckd_update($node, $msg);
-
-          if ( defined $node->{$self->_position_field}) {
-              $self->_update_children_position(@{$parent->{$key_children}});
-              $self->_update_children_position(@{$new_parent->{$key_children}});
-          }
+          #$self->_bckd_update($node, $msg);
       }
       else {
           $node->{$k} = $datas->{$k};
@@ -688,7 +619,7 @@ TreePath - Simple Tree Path!
  $tp = TreePath->new( conf => 't/conf/treefromdbix.yml')
 
  # see t/conf/treepath.yml for hash structure
- $tp = TreePath->new( datas => $datas);
+ $tp = TreePath->new( conf => $datas);
 
  also see t/01-tpath.t
 
@@ -715,14 +646,15 @@ TreePath - Simple Tree Path!
  This is the root node ( a simple hashref )
  it has no parent.
      {
-       '1' => {
+       'Page_1' => {
                 'id' => '1',
                 'name' => '/',
                 'parent' => '0'
+                'source' => 'Page'
               }
      }
 
-  $A = $tp->search( { name => 'A' } );
+  $A = $tp->search( { name => 'A', source => 'Page' } );
   See the dump :
 
     {
@@ -732,12 +664,14 @@ TreePath - Simple Tree Path!
                         'id' => '3',
                         'name' => 'B',
                         'parent' => $VAR1
-                      },
+                        'source' => 'Page'
+                     },
                       {
                         'children' => 'ARRAY(0x2fd69b0)',
                         'id' => '7',
                         'name' => 'F',
                         'parent' => $VAR1
+                        'source' => 'Page'
                       }
                     ],
       'id' => '2',
@@ -748,33 +682,31 @@ TreePath - Simple Tree Path!
                                   ],
                     'id' => '1',
                     'name' => '/',
-                    'parent' => '0'
+                    'parent' => '0',
+                    'source' => 'Page'
                   }
+     'source' => 'Page'
     }
 
     => 'parent' is a reference on root node and 'children' is an array containing 2 nodes
 
-    => if nodes have 'type', it's possible to 'link' some 'type' of node on some others types
+    => if nodes have 'source', it's possible to 'link' some 'source' of node on some others sources
 
-       For example, A:(type1), B:(type1, parent:A), C:(type2, parent:A) becomes :
-       A:(type1, children:[B], children_type2[C])
-
-       Each children can be ordered : { name => A, position => 2};
-
-       It's also possible to block link with 'rules'. 'link' must be follow these rules to be applied.
+       For example, A:(source1), B:(source1, parent:A), C:(source2, parent:A) becomes :
+       A:(source1, children:[B], children_source2[C])
 
 =cut
 
 =head2 search (hashref)
 
  # in scalar context return the first result
- my $E = $tp->search( { name => 'E' } );
+ my $E = $tp->search( { name => 'E', source => 'Page' } );
 
  # return all result in array context
- my @allE = $tp->search( { name => 'E' } );
+ my @allE = $tp->search( { name => 'E', source => 'Page' } );
 
  # It is also possible to specify a particular field of a hash
- my $B = $tp->search( { name => 'B', 'parent.name' => 'A'} );
+ my $B = $tp->search( { name => 'B', source => 'Page', 'parent.name' => 'A'} );
 
 =cut
 
@@ -829,13 +761,10 @@ TreePath - Simple Tree Path!
 =head2 add ($node, $parent)
 
  # add the root
- $root = $tp->add({ name => '/'}, 0);
+ $root = $tp->add({ name => '/', source => 'Node' }, 0);
 
- # add a node beneath the parent at the last position.
- $Z = $tp->add({ name => 'Z' }, $parent);
-
- # or at given position
- $Z = $tp->add({ name => 'Z', position => 2 }, $parent);
+ # add a node beneath the parent.
+ $Z = $tp->add({ name => 'Z', source => 'Node' }, $parent);
 
 =cut
 
@@ -844,8 +773,6 @@ TreePath - Simple Tree Path!
  # update node with somes datas
  $Z = $tp->update($node, { name => 'new_name' });
 
- # it's possible to update position
- $X = $tp->update($node, { position => 2 });
 
 =cut
 
