@@ -1,5 +1,5 @@
 package TreePath::Backend::DBIx;
-$TreePath::Backend::DBIx::VERSION = '0.14';
+$TreePath::Backend::DBIx::VERSION = '0.15';
 use Moose::Role;
 use base 'DBIx::Class::Schema';
 use Carp qw/croak/;
@@ -37,7 +37,7 @@ has model_config => (
 has 'schema'     => (
                      is        => 'rw',
                      predicate => 'has_schema',
-#                     lazy_build      => 1,
+                     lazy_build      => 1,
                     );
 
 has '_source_name' => (
@@ -60,6 +60,20 @@ sub _build_model_config {
   return $model_config
 }
 
+sub _build_schema {
+  my $self = shift;
+  my $config       = $self->conf;
+
+  my($dsn, $user, $password, $allattrs) = $self->_connect_info;
+
+  my $schema_class =  $self->model_config->{schema_class};
+  eval "require $schema_class";
+  if( $@ ){
+    die("Cannot load $schema_class : $@");
+  }
+  return $schema_class->connect($dsn,$user,$password,$allattrs);
+}
+
 sub _connect_info {
   my $self = shift;
 
@@ -111,68 +125,127 @@ sub _connect_info {
 }
 
 
+
 sub _load {
-  my $self = shift;
-
-  $self->_log("Loading tree from dbix");
-
-  my($dsn, $user, $password, $allattrs) = $self->_connect_info;
-
-  $self->_populate_backend($self->config->{backend}->{args}->{'populate_backend'})
-  if ( $self->can('_populate_backend') && ! defined $self->_populate_backend && defined $self->config->{backend}->{args}->{'populate_backend'} );
-
-  my $schema_class =  $self->model_config->{schema_class};
-  eval "require $schema_class";
-  if( $@ ){
-    die("Cannot load $schema_class : $@");
-  }
-  my $schema = $schema_class->connect($dsn,$user,$password,$allattrs);
-  my $source_name = $self->config->{backend}->{args}->{source_name};
-  $self->_source_name($source_name);
-  eval { $schema->resultset($source_name)->count };
-
-  if ( $@ ) {
-    print "Deploy and populate $dsn\n" if $self->debug;
-    $schema->deploy;
-    $schema->_populate if ( $schema->can('_populate') && $self->_populate_backend);
-  }
-  $self->schema($schema);
-
-  my @rs = $self->schema->resultset($source_name)->search();
-
-  my $search_field = $self->_search_field;
-  my $parent_field = $self->_parent_field;
-
-  return { map { $_->id => { name => $_->$search_field, parent => $_->$parent_field } } @rs};
-}
-
-
-sub _create {
     my $self = shift;
-    my $node = shift;
 
-    my $clone = $self->_clone_node($node);
-    $self->schema->resultset($self->_source_name)->create($clone);
+    $self->_log("Loading tree from dbix");
+
+    $self->_populate_backend($self->config->{backend}->{args}->{'populate_backend'})
+        if ( $self->can('_populate_backend') && ! defined $self->_populate_backend && defined $self->config->{backend}->{args}->{'populate_backend'} );
+
+    my $schema = $self->schema;
+    my @sources_name = keys %{$self->config->{backend}->{args}->{sources_name}}
+        or croak "'backend/args/sources_name' is not defined in conf file !";
+
+    my $datas = {};
+   foreach my $source_name ( @sources_name ) {
+
+       my($dsn, $user, $password, $allattrs) = $self->_connect_info;
+       $self->_source_name($source_name);
+       eval { $schema->resultset($source_name)->count };
+
+      if ( $@ ) {
+          print "Deploy and populate $dsn\n" if $self->debug;
+          $schema->deploy;
+          $schema->_populate if ( $schema->can('_populate') && $self->_populate_backend);
+      }
+      $self->schema($schema);
+      my $rs = $self->schema->resultset($source_name)->search();
+
+       my @primary_columns = $rs->result_source->primary_columns;
+       croak "Multi-column primary keys are not supported." if (scalar @primary_columns > 1 );
+       croak "No primary key found." if (scalar @primary_columns == 0 );
+       my $primary_key = shift @primary_columns;
+
+       foreach my $row ( $rs->all) {
+           my $obj = $self->_row_to_obj($row);
+           $obj->{$primary_key} = $row->$primary_key;
+           $datas->{$self->_obj_key($row)} = $obj;
+       }
+   }
+    return $datas;
 }
 
-sub _update {
+
+sub _row_to_obj {
     my $self = shift;
-    my $node = shift;
+    my $row  = shift;
 
-    my $clone = $self->_clone_node($node);
-    $self->schema->resultset($self->_source_name)->update_or_create($clone);
-}
 
-sub _delete {
-    my $self  = shift;
-    my $nodes = shift;
+    my $result_source = ref($row);
+    my ($source) = ($result_source =~ /^.*::(.*)/s);
 
-    foreach my $node (@$nodes) {
-        $self->schema->resultset($self->_source_name)->find($node->{id})->delete;
+    my $obj = {};
+
+    my $cols = $self->config->{backend}->{args}->{sources_name}->{$source}->{columns}
+        or croak "backend/args/sources_name/$source/columns is not defined in conf file !";
+
+    foreach my $col (@$cols) {
+
+        my $value = $row->$col;
+
+        # it's not a relationship'
+        if ( ! ref($value)) {
+            $obj->{$col} = $value;
+        }
+        # relationship has_many
+        elsif ( ref($value) eq 'DBIx::Class::ResultSet') {
+            my @results = $row->$col;
+            $obj->{$col} = [];
+            #  save rs in array
+            foreach my $res (@results) {
+                push( $obj->{$col}, $self->_obj_key($res));
+            }
+        }
+        # relationship belongs_to
+        else {
+            # save result in hashref
+            $obj->{$col} = { $self->_obj_key($row->$col) => 1 };
+        }
     }
+    # Ajout de la source a l'objet
+    $obj->{source} = $source;
+    return $obj;
 }
 
 
+sub _obj_key {
+    my $self = shift;
+    my $row  = shift;
+
+    my $result_source = ref($row);
+    my ($source) = ($result_source =~ /^.*::(.*)/s);
+    return $source . '_' . $row->id ;
+}
+
+
+# sub _create {
+#     my $self = shift;
+#     my $node = shift;
+
+#     my $clone = $self->_clone_node($node);
+#     $self->schema->resultset($self->_source_name)->create($clone);
+# }
+
+# sub _update {
+#     my $self = shift;
+#     my $node = shift;
+
+#     my $clone = $self->_clone_node($node);
+#     $self->schema->resultset($self->_source_name)->update_or_create($clone);
+# }
+
+# sub _delete {
+#     my $self  = shift;
+#     my $nodes = shift;
+
+#     foreach my $node (@$nodes) {
+#         $self->schema->resultset($self->_source_name)->find($node->{id})->delete;
+#     }
+# }
+
+
 
 =head1 NAME
 
@@ -180,7 +253,7 @@ TreePath::Backend::DBIx - Backend 'DBIx' for TreePath
 
 =head1 VERSION
 
-version 0.14
+version 0.15
 
 =head1 CONFIGURATION
 
@@ -198,9 +271,19 @@ version 0.14
              name: DBIx
              args:
                model: Model::TPath
-               source_name: Page
-               search_field: name
-               parent_field: parent_id
+               populate_backend: 1
+               sources_name:
+                 Page:
+                   columns:
+                     - name
+                     - parent
+                   search_field: name
+                   parent_field: parent
+
+                 Comment:
+                   columns:
+                     - page
+                   parent_field: page
 
 
 =head2 REQUIRED SCHEMA
