@@ -73,6 +73,17 @@ __PACKAGE__->set_primary_key("id");
 __PACKAGE__->add_unique_constraint("name_parent_unique", ["name", "parent_id"]);
 
 
+=head1 UNIQUE CONSTRAINTS
+
+=head2 C<name_parent_unique>
+
+=cut
+
+__PACKAGE__->belongs_to(
+  "parent",
+  "Schema::TPath::Result::Page",
+  { id => "parent_id" },
+);
 
 __PACKAGE__->meta->make_immutable;
 
